package xyz.brassgoggledcoders.boilerplate.tileentities;

import net.minecraft.entity.Entity;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.math.AxisAlignedBB;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.world.World;
import net.minecraftforge.fluids.*;
import xyz.brassgoggledcoders.boilerplate.utils.Selectors;

import java.util.List;

public abstract class TileEntityFluidBase extends TileEntityBase implements IFluidHandler
{
	private FluidTank tank;

	public TileEntityFluidBase()
	{
		tank = new FluidTank(getCapacity());
	}

	public abstract int getCapacity();

	public abstract int getTransferRate();

	public boolean pullFluidFrom(EnumFacing enumFacing)
	{
		IFluidHandler fluidHandler = this.getFluidHandlerForTransfer(enumFacing);
		boolean dirty = false;
		if(getTank().getFluidAmount() < getTank().getCapacity())
		{
			if(fluidHandler != null)
			{
				if(getTank().getFluidAmount() > 0)
				{
					FluidStack canPull = getTank().getFluid().copy();
					canPull.amount = getTank().getCapacity() - getTank().getFluidAmount();
					canPull.amount = Math.min(canPull.amount, getTransferRate());
					FluidStack drained = fluidHandler.drain(enumFacing.func_176734_d(), canPull, true);
					if(drained != null && drained.amount > 0)
					{
						getTank().fill(drained, true);
						dirty = true;
					}
				} else
				{
					FluidTankInfo[] infos = fluidHandler.getTankInfo(enumFacing.func_176734_d());
					if(infos != null)
					{
						for (FluidTankInfo info : infos) {
							if(info.fluid != null && info.fluid.amount > 0)
							{
								if(canFill(enumFacing, info.fluid.getFluid()))
								{
									FluidStack canPull = info.fluid.copy();
									canPull.amount = Math.min(getTransferRate(), canPull.amount);
									FluidStack drained = fluidHandler.drain(enumFacing.func_176734_d(), canPull, true);
									if(drained != null && drained.amount > 0)
									{
										getTank().fill(drained, true);
										dirty = true;
									}
								}
							}
						}
					}
				}
			}
		}

		return dirty;
	}

	public boolean pushFluidTo(EnumFacing facing)
	{
		IFluidHandler fluidHandler = this.getFluidHandlerForTransfer(facing);
		boolean dirty = false;
		if(getTank().getFluidAmount() > 0) {
			if(fluidHandler != null) {
				if(fluidHandler.canFill(facing.func_176734_d(), getTank().getFluid().getFluid())) {
					FluidStack push = getTank().getFluid().copy();
					push.amount = Math.min(push.amount, getTransferRate());
					int filled = fluidHandler.fill(facing.func_176734_d(), push, true);
					if(filled > 0) {
						getTank().drain(filled, true);
						dirty = true;
					}
				}
			}
		}
		return dirty;
	}

	@Override
	public int fill(EnumFacing from, FluidStack resource, boolean doFill)
	{
		return getTank().fill(resource, doFill);
	}

	@Override
	public FluidStack drain(EnumFacing from, FluidStack resource, boolean doDrain)
	{
		if (resource == null || !resource.isFluidEqual(getTank().getFluid()))
		{
			return null;
		}
		return getTank().drain(resource.amount, doDrain);
	}

	@Override
	public FluidStack drain(EnumFacing from, int maxDrain, boolean doDrain)
	{
		return getTank().drain(maxDrain, doDrain);
	}

	@Override
	public boolean canFill(EnumFacing from, Fluid fluid)
	{
		return true;
	}

	@Override
	public boolean canDrain(EnumFacing from, Fluid fluid)
	{
		return true;
	}

	@Override
	public FluidTankInfo[] getTankInfo(EnumFacing from)
	{
		return new FluidTankInfo[] { getTank().getInfo() };
	}

	@Override
	public void func_145839_a(NBTTagCompound nbtTagCompound)
	{
		super.func_145839_a(nbtTagCompound);
		if(getTank() == null)
		{
			tank = new FluidTank(getCapacity());
		}
		getTank().readFromNBT(nbtTagCompound);
	}

	@Override
	public NBTTagCompound func_189515_b(NBTTagCompound nbtTagCompound)
	{
		nbtTagCompound = super.func_189515_b(nbtTagCompound);
		nbtTagCompound = getTank().writeToNBT(nbtTagCompound);
		return nbtTagCompound;
	}

	public FluidTank getTank()
	{
		return tank;
	}

	public IFluidHandler getFluidHandlerForTransfer(EnumFacing enumfacing)
	{
		return getFluidHandlerAtPosition(this.func_145831_w(), func_174877_v().func_177972_a(enumfacing));
	}

	public IFluidHandler getFluidHandlerAtPosition(World world, BlockPos blockPos)
	{
		IFluidHandler fluidHandler = null;
		TileEntity tileEntity = world.func_175625_s(blockPos);

		if (tileEntity != null)
		{
			if (tileEntity instanceof IFluidHandler)
			{
				fluidHandler = (IFluidHandler)tileEntity;
			}
		}

		if (fluidHandler == null)
		{
			float sensitivity = 0.49f;
			List<Entity> list = world.func_175674_a(null, new AxisAlignedBB(
					blockPos.func_177958_n() + sensitivity, blockPos.func_177956_o() + sensitivity, blockPos.func_177952_p() + sensitivity,
					blockPos.func_177958_n() + 1 - sensitivity, blockPos.func_177956_o() + 1 - sensitivity, blockPos.func_177952_p() + 1 - sensitivity),
					Selectors.IFLUID_HANDLER_ENTITIES);

			if (list.size() > 0)
			{
				fluidHandler = (IFluidHandler)list.get(world.field_73012_v.nextInt(list.size()));
			}
		}

		return fluidHandler;
	}
}
